==> deepchecks_llm_client-0.23.7.tar.gz: deepchecks_llm_client-0.23.7/deepchecks_llm_client/data_types.py <==
import enum
import logging
from dataclasses import dataclass

__all__ = ["EnvType", "AnnotationType", "Interaction", "Step", "StepType", "Application",
           "ApplicationType", "ApplicationVersion", "ApplicationVersionSchema",
           "LogInteraction", "LogInteractionType",
           "PropertyColumnType", "CustomPropertyType", "InteractionCompleteEvents"]

from datetime import datetime
from typing import Any, Dict, List, Optional, Union

import pytz

logging.basicConfig()
logger = logging.getLogger(__name__)


class EnvType(str, enum.Enum):
    PROD = "PROD"
    EVAL = "EVAL"
    PENTEST = "PENTEST"


class AnnotationType(str, enum.Enum):
    GOOD = "good"
    BAD = "bad"
    UNKNOWN = "unknown"


class PropertyColumnType(str, enum.Enum):
    CATEGORICAL = "categorical"
    NUMERIC = "numeric"


@dataclass
class Interaction:
    user_interaction_id: str
    input: str
    information_retrieval: str
    history: str
    full_prompt: str
    output: str
    topic: str
    output_properties: Dict[str, Any]
    input_properties: Dict[str, Any]
    custom_properties: Dict[str, Any]
    llm_properties: Dict[str, Any]
    llm_properties_reasons: Dict[str, Any]
    created_at: datetime
    interaction_datetime: datetime
    is_completed: bool
    interaction_type: str
    session_id: str


class StepType(str, enum.Enum):
    LLM = "LLM"
    INFORMATION_RETRIEVAL = "INFORMATION_RETRIEVAL"
    TRANSFORMATION = "TRANSFORMATION"
    FILTER = "FILTER"
    FINE_TUNING = "FINE_TUNING"
    PII_REMOVAL = "PII_REMOVAL"
    UDF = "UDF"


@dataclass
class Step:
    name: str
    output: str
    type: Union[StepType, None] = None
    attributes: Union[Dict[str, Any], None] = None
    started_at: Union[datetime, float, None] = None
    annotation: Union[AnnotationType, str, None] = None
    finished_at: Union[datetime, float, None] = None
    input: Union[str, None] = None
    error: Union[str, None] = None

    def to_json(self):
        return {
            "name": self.name,
            "annotation": (
                None if self.annotation is None else
                self.annotation.value if isinstance(self.annotation, AnnotationType) else self.annotation.lower().strip()
            ),
            "type": self.type.value if isinstance(self.type, StepType) else self.type,
            "attributes": self.attributes,
            "started_at": self.started_at.timestamp() if isinstance(self.started_at, datetime) else self.started_at,
            "finished_at": self.finished_at.timestamp() if isinstance(self.finished_at, datetime) else self.finished_at,
            "input": self.input,
            "output": self.output,
            "error": self.error,
        }

    @classmethod
    def as_jsonl(cls, steps):
        if steps is None:
            return None
        return [step.to_json() for step in steps]


@dataclass
class LogInteraction:
    """A dataclass representing an interaction.

    Attributes
    ----------
    input : str
        Input data
    output : str
        Output data
    full_prompt : str, optional
        Full prompt data, defaults to None
    annotation : AnnotationType, optional
        Annotation type of the interaction, defaults to None
    user_interaction_id : str, optional
        Unique identifier of the interaction, defaults to None
    steps : list of Step, optional
        List of steps taken during the interaction, defaults to None
    custom_props : dict, optional
        Additional custom properties, defaults to None
    information_retrieval : str, optional
        Information retrieval, defaults to None
    history : str, optional
        History (for instance "chat history"), defaults to None
    annotation_reason : str, optional
        Reason for the annotation, defaults to None
    started_at : datetime or float, optional
        Timestamp the interaction started at. Datetime format is deprecated, use timestamp instead
    finished_at : datetime or float, optional
        Timestamp the interaction finished at. Datetime format is deprecated, use timestamp instead
    vuln_type : str, optional
        Type of vulnerability (Only used in case of EnvType.PENTEST and must be sent there), defaults to None
    vuln_trigger_str : str, optional
        Vulnerability trigger string (Only used in case of EnvType.PENTEST and is optional there), defaults to None
    session_id: str, optional
        The identifier for the session associated with this interaction.
        If not provided, a session ID will be automatically generated.
    interaction_type: str, optional
        The type of interaction.
        None is deprecated. If not provided, the interaction type will default to the application’s default type.
    """

    input: Optional[str] = None
    output: Optional[str] = None
    full_prompt: Optional[str] = None
    annotation: Optional[Union[AnnotationType, str]] = None
    user_interaction_id: Optional[str] = None
    steps: Optional[List[Step]] = None
    custom_props: Optional[Dict[str, Any]] = None
    information_retrieval: Optional[Union[str, List[str]]] = None
    history: Optional[Union[str, List[str]]] = None
    annotation_reason: Optional[str] = None
    started_at: Optional[Union[datetime, float]] = None
    finished_at: Optional[Union[datetime, float]] = None
    vuln_type: Optional[str] = None
    vuln_trigger_str: Optional[str] = None
    topic: Optional[str] = None
    is_completed: bool = True
    interaction_type: Optional[str] = None
    session_id: Optional[str] = None

    def to_json(self):
        if isinstance(self.started_at, datetime) or isinstance(self.finished_at, datetime):
            logger.warning(
                "Deprecation Warning: Usage of datetime for started_at/finished_at is deprecated, use timestamp instead."
            )
            self.started_at = self.started_at.timestamp() if self.started_at else datetime.now(tz=pytz.UTC).timestamp()
            self.finished_at = self.finished_at.timestamp() if self.finished_at else None
        if self.interaction_type is None:
            logger.warning(
                "Deprecation Warning: The value 'None' for 'interaction_type' is deprecated. "
                "Please specify an explicit interaction type."
            )

        data = {
            "input": self.input,
            "output": self.output,
            "full_prompt": self.full_prompt,
            "information_retrieval": self.information_retrieval \
                if self.information_retrieval is None or isinstance(self.information_retrieval, list) \
                else [self.information_retrieval],
            "history": self.history \
                if self.history is None or isinstance(self.history, list) \
                else [self.history],
            "annotation": self.annotation.value if isinstance(self.annotation, AnnotationType) else self.annotation,
            "user_interaction_id": self.user_interaction_id,
            "steps": [step.to_json() for step in self.steps] if self.steps else None,
            "custom_props": self.custom_props,
            "annotation_reason": self.annotation_reason,
            "vuln_type": self.vuln_type,
            "vuln_trigger_str": self.vuln_trigger_str,
            "started_at": self.started_at,
            "finished_at": self.finished_at,
            "is_completed": self.is_completed,
            "session_id": self.session_id,
            "interaction_type": self.interaction_type,
        }
        if self.topic is not None:
            data["topic"] = self.topic

        return data


@dataclass
class LogInteractionType(LogInteraction):
    pass


@dataclass
class CustomPropertyType:
    display_name: str
    type: PropertyColumnType
    description: str


class ApplicationType(str, enum.Enum):
    QA = "Q&A"
    OTHER = "OTHER"
    SUMMARIZATION = "SUMMARIZATION"
    GENERATION = "GENERATION"
    CLASSIFICATION = "CLASSIFICATION"
    FEATURE_EXTRACTION = "FEATURE_EXTRACTION"


class BuiltInInteractionType(str, enum.Enum):
    QA = "Q&A"
    OTHER = "Other"
    SUMMARIZATION = "Summarization"
    CLASSIFICATION = "Classification"
    GENERATION = "Generation"
    FEATURE_EXTRACTION = "Feature_Extraction"


class InteractionCompleteEvents(str, enum.Enum):
    TOPICS_COMPLETED = "topics_completed"
    PROPERTIES_COMPLETED = "properties_completed"
    SIMILARITY_COMPLETED = "similarity_completed"
    LLM_PROPERTIES_COMPLETED = "llm_properties_completed"
    ANNOTATION_COMPLETED = "annotation_completed"
    DC_EVALUATION_COMPLETED = "dc_evaluation_completed"


@dataclass
class ApplicationVersionSchema:
    name: str
    description: Optional[str] = None
    additional_fields: Optional[Dict[str, Any]] = None

    def to_json(self):
        return {
            "name": self.name,
            "description": self.description,
            "custom": [{key: value} for key, value in self.additional_fields.items()] if self.additional_fields else []
        }


@dataclass
class ApplicationVersion:
    id: int
    name: str
    ai_model: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    custom: Optional[List[Dict[str, Any]]] = None


@dataclass
class Application:
    id: int
    name: str
    kind: ApplicationType
    created_at: datetime
    updated_at: datetime
    in_progress: bool
    versions: List[ApplicationVersion]
    interaction_types: List[str]
    description: Optional[str] = None
    log_latest_insert_time_epoch: Optional[int] = None
    n_of_llm_properties: Optional[int] = None
    n_of_interactions: Optional[int] = None
    notifications_enabled: Optional[bool] = None


APP_KIND_TO_INTERACTION_TYPE = {
    ApplicationType.QA: BuiltInInteractionType.QA,
    ApplicationType.OTHER: BuiltInInteractionType.OTHER,
    ApplicationType.SUMMARIZATION: BuiltInInteractionType.SUMMARIZATION,
    ApplicationType.CLASSIFICATION: BuiltInInteractionType.CLASSIFICATION,
    ApplicationType.GENERATION: BuiltInInteractionType.GENERATION,
    ApplicationType.FEATURE_EXTRACTION: BuiltInInteractionType.FEATURE_EXTRACTION,
}

==> deepchecks_llm_client-0.23.18.tar.gz: deepchecks_llm_client-0.23.18/deepchecks_llm_client/data_types.py <==
import enum
import logging
from dataclasses import dataclass

__all__ = ["EnvType", "AnnotationType", "Interaction", "Step", "StepType", "Application",
           "ApplicationType", "ApplicationVersion", "ApplicationVersionSchema",
           "LogInteraction", "LogInteractionType",
           "PropertyColumnType", "CustomPropertyType", "InteractionCompleteEvents"]

from datetime import datetime
from typing import Any, Dict, List, Optional, Union

import pytz

logging.basicConfig()
logger = logging.getLogger(__name__)


class EnvType(str, enum.Enum):
    PROD = "PROD"
    EVAL = "EVAL"
    PENTEST = "PENTEST"


class AnnotationType(str, enum.Enum):
    GOOD = "good"
    BAD = "bad"
    UNKNOWN = "unknown"


class PropertyColumnType(str, enum.Enum):
    CATEGORICAL = "categorical"
    NUMERIC = "numeric"


@dataclass
class Interaction:
    user_interaction_id: str
    input: str
    information_retrieval: str
    history: str
    full_prompt: str
    output: str
    topic: str
    output_properties: Dict[str, Any]
    input_properties: Dict[str, Any]
    custom_properties: Dict[str, Any]
    llm_properties: Dict[str, Any]
    llm_properties_reasons: Dict[str, Any]
    created_at: datetime
    interaction_datetime: datetime
    is_completed: bool
    interaction_type: str
    session_id: str


class StepType(str, enum.Enum):
    LLM = "LLM"
    INFORMATION_RETRIEVAL = "INFORMATION_RETRIEVAL"
    TRANSFORMATION = "TRANSFORMATION"
    FILTER = "FILTER"
    FINE_TUNING = "FINE_TUNING"
    PII_REMOVAL = "PII_REMOVAL"
    UDF = "UDF"


@dataclass
class Step:
    name: str
    output: str
    type: Union[StepType, None] = None
    attributes: Union[Dict[str, Any], None] = None
    started_at: Union[datetime, float, None] = None
    annotation: Union[AnnotationType, str, None] = None
    finished_at: Union[datetime, float, None] = None
    input: Union[str, None] = None
    error: Union[str, None] = None

    def to_json(self):
        return {
            "name": self.name,
            "annotation": (
                None if self.annotation is None else
                self.annotation.value if isinstance(self.annotation, AnnotationType) else self.annotation.lower().strip()
            ),
            "type": self.type.value if isinstance(self.type, StepType) else self.type,
            "attributes": self.attributes,
            "started_at": self.started_at.timestamp() if isinstance(self.started_at, datetime) else self.started_at,
            "finished_at": self.finished_at.timestamp() if isinstance(self.finished_at, datetime) else self.finished_at,
            "input": self.input,
            "output": self.output,
            "error": self.error,
        }

    @classmethod
    def as_jsonl(cls, steps):
        if steps is None:
            return None
        return [step.to_json() for step in steps]


@dataclass
class LogInteraction:
    """A dataclass representing an interaction.

    Attributes
    ----------
    input : str
        Input data
    output : str
        Output data
    full_prompt : str, optional
        Full prompt data, defaults to None
    annotation : AnnotationType, optional
        Annotation type of the interaction, defaults to None
    user_interaction_id : str, optional
        Unique identifier of the interaction, defaults to None
    steps : list of Step, optional
        List of steps taken during the interaction, defaults to None
    custom_props : dict, optional
        Additional custom properties, defaults to None
    information_retrieval : str, optional
        Information retrieval, defaults to None
    history : str, optional
        History (for instance "chat history"), defaults to None
    annotation_reason : str, optional
        Reason for the annotation, defaults to None
    started_at : datetime or float, optional
        Timestamp the interaction started at. Datetime format is deprecated, use timestamp instead
    finished_at : datetime or float, optional
        Timestamp the interaction finished at. Datetime format is deprecated, use timestamp instead
    vuln_type : str, optional
        Type of vulnerability (Only used in case of EnvType.PENTEST and must be sent there), defaults to None
    vuln_trigger_str : str, optional
        Vulnerability trigger string (Only used in case of EnvType.PENTEST and is optional there), defaults to None
    session_id: str, optional
        The identifier for the session associated with this interaction.
        If not provided, a session ID will be automatically generated.
    interaction_type: str, optional
        The type of interaction.
        None is deprecated. If not provided, the interaction type will default to the application’s default type.
    """

    input: Optional[str] = None
    output: Optional[str] = None
    full_prompt: Optional[str] = None
    annotation: Optional[Union[AnnotationType, str]] = None
    user_interaction_id: Optional[Union[str, int]] = None
    steps: Optional[List[Step]] = None
    custom_props: Optional[Dict[str, Any]] = None
    information_retrieval: Optional[Union[str, List[str]]] = None
    history: Optional[Union[str, List[str]]] = None
    annotation_reason: Optional[str] = None
    started_at: Optional[Union[datetime, float]] = None
    finished_at: Optional[Union[datetime, float]] = None
    vuln_type: Optional[str] = None
    vuln_trigger_str: Optional[str] = None
    topic: Optional[str] = None
    is_completed: bool = True
    interaction_type: Optional[str] = None
    session_id: Optional[Union[str, int]] = None

    def to_json(self):
        if isinstance(self.started_at, datetime) or isinstance(self.finished_at, datetime):
            logger.warning(
                "Deprecation Warning: Usage of datetime for started_at/finished_at is deprecated, use timestamp instead."
            )
            self.started_at = self.started_at.timestamp() if self.started_at else datetime.now(tz=pytz.UTC).timestamp()
            self.finished_at = self.finished_at.timestamp() if self.finished_at else None
        if self.interaction_type is None:
            logger.warning(
                "Deprecation Warning: The value 'None' for 'interaction_type' is deprecated. "
                "Please specify an explicit interaction type."
            )

        data = {
            "input": self.input,
            "output": self.output,
            "full_prompt": self.full_prompt,
            "information_retrieval": self.information_retrieval \
                if self.information_retrieval is None or isinstance(self.information_retrieval, list) \
                else [self.information_retrieval],
            "history": self.history \
                if self.history is None or isinstance(self.history, list) \
                else [self.history],
            "annotation": (
                None if self.annotation is None else
                self.annotation.value if isinstance(self.annotation, AnnotationType)
                else str(self.annotation).lower().strip()
            ),
            "user_interaction_id": str(self.user_interaction_id) if self.user_interaction_id is not None else None,
            "steps": [step.to_json() for step in self.steps] if self.steps else None,
            "custom_props": self.custom_props,
            "annotation_reason": self.annotation_reason,
            "vuln_type": self.vuln_type,
            "vuln_trigger_str": self.vuln_trigger_str,
            "started_at": self.started_at,
            "finished_at": self.finished_at,
            "is_completed": self.is_completed,
            "session_id": str(self.session_id) if self.session_id is not None else None,
            "interaction_type": self.interaction_type,
        }
        if self.topic is not None:
            data["topic"] = self.topic

        return data


@dataclass
class LogInteractionType(LogInteraction):
    pass


@dataclass
class CustomPropertyType:
    display_name: str
    type: PropertyColumnType
    description: str


class ApplicationType(str, enum.Enum):
    QA = "Q&A"
    OTHER = "OTHER"
    SUMMARIZATION = "SUMMARIZATION"
    GENERATION = "GENERATION"
    CLASSIFICATION = "CLASSIFICATION"
    FEATURE_EXTRACTION = "FEATURE EXTRACTION"


class BuiltInInteractionType(str, enum.Enum):
    QA = "Q&A"
    OTHER = "Other"
    SUMMARIZATION = "Summarization"
    CLASSIFICATION = "Classification"
    GENERATION = "Generation"
    FEATURE_EXTRACTION = "Feature Extraction"


class InteractionCompleteEvents(str, enum.Enum):
    TOPICS_COMPLETED = "topics_completed"
    PROPERTIES_COMPLETED = "properties_completed"
    SIMILARITY_COMPLETED = "similarity_completed"
    LLM_PROPERTIES_COMPLETED = "llm_properties_completed"
    ANNOTATION_COMPLETED = "annotation_completed"
    DC_EVALUATION_COMPLETED = "dc_evaluation_completed"


@dataclass
class ApplicationVersionSchema:
    name: str
    description: Optional[str] = None
    additional_fields: Optional[Dict[str, Any]] = None

    def to_json(self):
        return {
            "name": self.name,
            "description": self.description,
            "custom": [{key: value} for key, value in self.additional_fields.items()] if self.additional_fields else []
        }


@dataclass
class ApplicationVersion:
    id: int
    name: str
    ai_model: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    custom: Optional[List[Dict[str, Any]]] = None


@dataclass
class Application:
    id: int
    name: str
    kind: ApplicationType
    created_at: datetime
    updated_at: datetime
    in_progress: bool
    versions: List[ApplicationVersion]
    interaction_types: List[str]
    description: Optional[str] = None
    log_latest_insert_time_epoch: Optional[int] = None
    n_of_llm_properties: Optional[int] = None
    n_of_interactions: Optional[int] = None
    notifications_enabled: Optional[bool] = None


APP_KIND_TO_INTERACTION_TYPE = {
    ApplicationType.QA: BuiltInInteractionType.QA,
    ApplicationType.OTHER: BuiltInInteractionType.OTHER,
    ApplicationType.SUMMARIZATION: BuiltInInteractionType.SUMMARIZATION,
    ApplicationType.CLASSIFICATION: BuiltInInteractionType.CLASSIFICATION,
    ApplicationType.GENERATION: BuiltInInteractionType.GENERATION,
    ApplicationType.FEATURE_EXTRACTION: BuiltInInteractionType.FEATURE_EXTRACTION,
}
